'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var UglifyJS = require('uglify-js');

// @ts-nocheck
const formatSourceDataStr = `function formatSourceData() {
  return {
    cookie: document.cookie.split("; ").reduce(function (result, cookie) {
      const [name, value] = cookie.split("=");
      result[name] = decodeURIComponent(value);
      return result;
    }, {}),
    query: location.search
      .slice(1)
      .split("&")
      .reduce(function (result, item) {
        var parts = item.split("=");
        result[decodeURIComponent(parts[0])] = decodeURIComponent(
          parts[1] || ""
        );
        return result;
      }, {}),
    hash: location.hash.slice(2),
    path: location.pathname,
  };
}`;

const PREFETCH = '__PREFETCH__';

// 由于 generateHash 存在递归调用，在解析自己函数名时会出现异常，所以直接字符串声明一下
const generateHashStr = `function generateHash(obj) {
  return Object.keys(obj).sort().reduce(function(hash, k) {
    const v = obj[k];
    if (typeof v === 'undefined') { return hash; }
    const str = \`\${k}:\${typeof v === 'object' ? generateHash(v) : v}\`;
    for(let i = 0; i < str.length; i++) {
        hash = (hash << 5) - hash + str.charCodeAt(i);
    }
    return hash
  }, 0).toString(36);
}`;

const setCacheStr = `function setCache(p, o, c) {
  const key = generateHash(o);
  window[PREFETCH] = window[PREFETCH] || {};
  window[PREFETCH][key] = {
    count: c || 1,
    value: p,
  };
}`;

/**
 * 将 object 序列化为字符串模板
 * @todo 改为抽象语法树进行格式化生成
 * @param config 配置参数，包含函数类型
 * @returns
 */
function stringifyConfig(config) {
    return `{${Object.entries(config)
        .map(([key, val]) => {
        let valStr = val;
        if (typeof val === "string") {
            valStr = `"${val}"`;
        }
        else if (typeof val === "function") {
            valStr = val.toString();
        }
        return `${key}:${valStr}`;
    })
        .join(",")}}`;
}
function stringify(config) {
    const list = Array.isArray(config) ? config : [config];
    return `[${list.map(stringifyConfig).join(",")}]`;
}
// export function stringify(json) {
//   const k = "$_$";
//   const tmp = JSON.stringify(
//     json,
//     (_key, val) => {
//       return typeof val === "function" ? `${k}${val.toString()}${k}` : val;
//     },
//     2
//   );
//   console.log(tmp);
//   return tmp
//     .replaceAll(`"${k}`, "")
//     .replaceAll(`${k}"`, "")
//     .replaceAll("\\", "");
// }

function formatParams(config, data) {
    const { url, method } = config;
    try {
        const { adapter } = config;
        const { body, header, query = {} } = adapter(data) || {};
        const q = Object.keys(query)
            .map((key) => `${key}=${query[key]}`)
            .join("&");
        return { url: q ? `${url}?${q}` : url, method, body, header };
    }
    catch (error) {
        return { url, method };
    }
}
function common(list, request) {
    return `;(function SetupPrefetch(list) {
try {
  const PREFETCH = "${PREFETCH}";
  const sourceData = (${formatSourceDataStr})();
  ${generateHashStr}
  ${setCacheStr}
  ${request.toString()}
  ${formatParams.toString()}
  function send(config) {
    const params = formatParams(config, sourceData);
    const { url, ...options } = params;
    setCache(request(url, options), params, config.count || 1);
  }
  ${stringify(list)}.forEach(({ trigger, ...item }) => {
    if ((typeof trigger === 'function' && trigger(sourceData)) || !trigger) {
      send(item);
    }
  });
} catch(e) {
  console.error('[prefetch-api error]', String(e));
}
})()`;
}

// @ts-nocheck
function fetchTemplate(list) {
    // 请求适配器，必须命名为request
    function request(url, options) {
        const { header, method = "get" } = options;
        const headers = new Headers();
        headers.append("Content-Type", "application/json");
        if (header) {
            Object.keys(header).forEach(function (key) {
                headers.append(key, header[key]);
            });
        }
        const body = options.body ? JSON.stringify(options.body) : undefined;
        const params = {
            body,
            headers,
            method: method.toLocaleUpperCase(),
        };
        return fetch(url, params).then((res) => res.ok
            ? {
                data: res.json(),
                status: res.status,
                statusText: res.statusText,
                headers: {},
                config: {},
                request: res,
            }
            : Promise.reject({ status: res.status, statusText: res.statusText }), () => Promise.rejects({ status: res.status, statusText: res.statusText }));
    }
    return common(list, request);
}

// @ts-nocheck
function xhrTemplate(list) {
    // 请求适配器，必须命名为request
    function request(url, options) {
        return new Promise(function (resolve, reject) {
            const { method, header, body } = options;
            let xhr = new XMLHttpRequest();
            xhr.open(method, url);
            xhr.onload = function onload() {
                if (xhr.status >= 200 && xhr.status < 300) {
                    resolve({
                        data: JSON.parse(xhr.responseText),
                        status: xhr.status,
                        statusText: xhr.statusText,
                        headers: {},
                        config: {},
                        request: xhr,
                    });
                }
                else {
                    reject({
                        status: xhr.status,
                        statusText: xhr.statusText,
                    });
                }
            };
            xhr.onerror = function onerror() {
                reject({
                    status: xhr.status,
                    statusText: xhr.statusText,
                });
            };
            xhr.setRequestHeader("Content-Type", "application/json");
            if (header) {
                Object.keys(header).forEach(function (key) {
                    xhr.setRequestHeader(key, header[key]);
                });
            }
            xhr.send(body);
        });
    }
    return common(list, request);
}

const defaultPluginConfig = {
    api: "fetch",
    list: [],
    minify: true,
};
function checkCount(count) {
    try {
        const c = parseInt(String(count));
        if (isNaN(c) || c < 1) {
            return 1;
        }
        return c;
    }
    catch (error) {
        return 1;
    }
}
function checkTrigger(trigger) {
    if (trigger && typeof trigger === "function") {
        return trigger;
    }
    // trigger === 0、false、
    if (trigger !== false && trigger !== null) {
        return undefined;
    }
}
function formatConfig(list) {
    return (list
        // trigger == true or trigger === undefined
        .filter((item) => item.trigger || item.trigger === undefined)
        .map((item) => {
        const trigger = checkTrigger(item.trigger);
        const config = Object.assign(Object.assign({}, item), { count: checkCount(item.count) });
        if (trigger) {
            config.trigger = trigger;
        }
        return config;
    }));
}
function formatPrefetchScript(config) {
    const { api = "fetch", minify = true } = config;
    const script = api === "fetch" ? fetchTemplate(config.list) : xhrTemplate(config.list);
    if (minify) {
        const result = UglifyJS.minify(script);
        if (!result.error) {
            return result.code;
        }
    }
    return script;
}
function PrefetchPlugin(userConfig) {
    const config = Object.assign({}, defaultPluginConfig, userConfig);
    config.list = formatConfig(config.list);
    return {
        name: "vite-plugin-prefetch",
        transformIndexHtml: {
            enforce: "post",
            transform() {
                if (Array.isArray(config.list) && config.list.length) {
                    return [
                        {
                            tag: "script",
                            children: formatPrefetchScript(config),
                            injectTo: "head",
                        },
                    ];
                }
            },
        },
    };
}

exports.PrefetchPlugin = PrefetchPlugin;
exports.default = PrefetchPlugin;
